defmodule Dlex.Ql do
  @moduledoc """
  Query language for DGraph

  allofterms*: `allofterms(name, "Michael”)`
  * *anyofterms*: `anyofterms(name, "foo bar")`
  * *alloftext*:  `alloftext(name@en, "the man runs")`  (fulltext index)
  * *eq* : equal to
  * *ge*: greater than or equal to
  * *le*: less than or equal to
  * *gt*: greater than
  * *lt*: less than

  {
  scott(func: eq(name@en, "Ridley Scott")) {
    name@en
    initial_release_date
    director.film @filter(le(initial_release_date, "2000")) {
      name@en
      initial_release_date
    }
  }
  }


  Repo.all(from u in User, where: u.age > ^age)

      name = "Ridley Scott"
      query

    quote do
      find anyofterms(name, ^name) do
        director.film do
          initial_release_date < 2000
        end
      end
    end

    quote do
      where name == ^name and type == User do
        director.film do
          where initial_release_date < 2000 and type == Film do
            director.foo do
              bar == "test"
            end
          end
        end
      end
    end

      Arango.Query.execute(
      fulltext: {"location", "alternatives", "prefix:"},
      filter: {"population", :!=, 0},
      limit: limit,
      sort: {:desc, "population"}
    )
  """

  def query(
        {:anyofterms, [],
         [
           {:name, [], Elixir},
           {:^, [], [{:name, [], Elixir}]},
           [
             do:
               {{:., [], [{:director, [], Elixir}, :film]}, [],
                [
                  [
                    do:
                      {:<, [context: Elixir, import: Kernel],
                       [{:initial_release_date, [], Elixir}, 2000]}
                  ]
                ]}
           ]
         ]}
      ) do
  end
end
